/**************************************************************************
 *  Copyright (c) 2002  Hartmut Ring, Universitaet Siegen                 *
 *  http://www.math.uni-siegen.de/ring                                    *
 *                                                                        *
 *  This file is part of the pariGUIde package.                           *
 *                                                                        *
 *  pariGUIde is free software; you can redistribute it and/or modify it  *
 *  under the terms of the GNU General Public License (GPL)               *
 *  as published by the Free Software Foundation:                         *
 *  http://www.opensource.org/licenses/gpl-license.html                   *
 *************************************************************************/
// editwindow.h

#ifndef __EDITWINDOW_H__
#define __EDITWINDOW_H__

#define PARI_GUI_VERSION "0.43"

#include <qwidget.h>
#include <qtextview.h>
#include <qmultilineedit.h>
#include <qsplitter.h>
#include <qtabwidget.h>
#include <qtextbrowser.h>
#include <qaction.h>

#include "gpcontroller.h"
#include "plotview.h"

#if QT_VERSION > 300
#include <qregexp.h>
#include <qtimer.h>
#endif

class MyQTabWidget: public QTabWidget {
  Q_OBJECT
public:
  MyQTabWidget (QWidget* parent) : QTabWidget(parent) {}
  virtual QSize sizeHint() const {return QSize(600,400);}
};

class EditWindow: public QSplitter {
  Q_OBJECT
public:
  EditWindow (QWidget* parent, GpController* pariGP);
  bool openDemo();
  bool demoStep();
public slots:
  void calc(); 
  void interrupt() {} // without this definition `MOC` stumbles
  void up();
  void down(); 
  void stop();
  void open();
  void save();
  void selectFont(); 
  void setPrecision();
  void insertSum();
  void insertProd();
  void insertIf();
  void insertTable();
  void insertPlot();
  void insertMatrix();
  void insertInt();
private slots:
  void newLine();
protected:
  //virtual void drawSplitter (QPainter*, QCOORD x, QCOORD y, QCOORD w, QCOORD h) {}
private:
  void setPrecision (int n);
  void dispatch (QString pariInput, QString& output, QString& error);
  void displayTimeAndType();
  void appendRecord (const QString& output);
  void scrollToEndOfRecord();
  void copyInput (int p); 
  void formatMatrix (QString& s, bool table=false);

  GpController*   _pari;
  MyQTabWidget*   _tabView;
  QTextView*      _recordWnd;
  QString         _record;
  int             _insertionPos;
  QMultiLineEdit* _inputWnd;

  PlotView*       _plotWnd;
  QTextBrowser*   _tutorialWnd;
  QTextBrowser*   _referenceWnd;
  bool            _firstInput;
  int             _realPrecision;
  int             _oldPrecision;
  QFont           _font;
  QString         _presentation;
  bool            _matrix;

#if (!defined KEIN_PATCH_WEGEN_QT_FEHLER) //no patch, because of an Qt-error
  QTimer*         _timer;
public slots:
  void scrollDown();
#endif
};

#endif //__EDITWINDOW_H__
