/**************************************************************************
 *  Copyright (c) 2002  Hartmut Ring, Universitaet Siegen                 *
 *  http://www.math.uni-siegen.de/ring                                    *
 *                                                                        *
 *  This file is part of the pariGUIde package.                           *
 *                                                                        *
 *  pariGUIde is free software; you can redistribute it and/or modify it  *
 *  under the terms of the GNU General Public License (GPL)               *
 *  as published by the Free Software Foundation:                         *
 *  http://www.opensource.org/licenses/gpl-license.html                   *
 *************************************************************************/
// gpcontroller.cpp

#include "gpcontroller.h"

GpController::GpController() {
  _buffer = new char[BUFSIZE];

#ifdef _WIN32
  _hPipeStdOutRead  = 0; // stdout of Pari
  _hPipeStdErrRead  = 0; // stderr of Pari
  _hPipeStdInWrite  = 0; // stdin  of Pari
  ZeroMemory (&_processInfo, sizeof(_processInfo));

  if (! CreatePariChildProcess())
    ::MessageBox (0, PARI_EXE tr("could not start"),
                 "pariGUIde", MB_OK | MB_ICONSTOP);
#endif

#ifdef _LINUX
  if ( CreatePariChildProcess()) {
    printf("\n");
    printf("PARI-GP not found!\n");
    printf("Please install it, or adjust your $PATH variable.\n");
    exit(-1);
  }
#endif

  writeStdIn ("print(\"\")\n1/0\n"); //say hello to PARI (provoke a failure)
  sleep(1000);

  while (! stdErrAvailable())
    ;
  readStdErr();
  readStdOut();
}

GpController::~GpController() {
   //quit pari polite
  writeStdIn ("\\q\n");
  sleep (200);

#ifdef _WIN32
  //kill pari
  TerminateProcess (_processInfo.hProcess, 0);
  Sleep (200);
  CloseHandle (_processInfo.hProcess);
  CloseHandle (_hPipeStdOutRead);
  CloseHandle (_hPipeStdErrRead);
  CloseHandle (_hPipeStdInWrite);
  delete[] _buffer;
#endif

#ifdef _LINUX
  kill(_paripid, 9);
#endif
}

////////////////////////////////////////////////////////////////////////
// start Pari-GP as invisible process and connect with pipes:
/* 
  pipes:
  PARI                                                               this program

  stdin     <----  hPipeStdInRead       <----  _hPipeStdInWrite
  stdout   ---->  hPipeStdOutWrite   ---->  _hPipeStdOutRead
  strderr  ---->  hPipeStdErrWrite     ---->  _hPipeStdErrRead
*/

#ifdef _WIN32
bool GpController::CreatePariChildProcess() {

  SECURITY_ATTRIBUTES sa = {0};
  STARTUPINFO         si = {0};
  sa.nLength = sizeof(sa);
  sa.bInheritHandle = TRUE;
  sa.lpSecurityDescriptor = NULL;

  HANDLE hPipeStdInRead, hPipeStdOutWrite, hPipeStdErrWrite;
  CreatePipe (&hPipeStdInRead,  &_hPipeStdInWrite,  &sa, 0);
  CreatePipe (&_hPipeStdOutRead, &hPipeStdOutWrite, &sa, 0);
  CreatePipe (&_hPipeStdErrRead, &hPipeStdErrWrite, &sa, 0);

  // connect PARI-GP as invisible child process with this pipes
  si.cb = sizeof(si);
  si.dwFlags     = STARTF_USESHOWWINDOW | STARTF_USESTDHANDLES;
  si.wShowWindow = SW_HIDE;
  si.hStdInput   = hPipeStdInRead;
  si.hStdOutput  = hPipeStdOutWrite;
  si.hStdError   = hPipeStdErrWrite;

  // find the absolute path to pari.exe and the config-file
  char pariEnv[MAX_PATH];
  char pariCurDir[MAX_PATH];
  GetModuleFileName (NULL, _pariExePath, MAX_PATH);
  lstrcpy (pariCurDir, _pariExePath);
  lstrcpy (pariEnv, "GPRC=");
  lstrcat (pariEnv, _pariExePath);

  char*
  p = strrchr (pariCurDir, DIR_SEPARATOR);   *p = '\0';
  p = strrchr (pariEnv, DIR_SEPARATOR);      lstrcpy (p+1, "parigp.cfg");
  p[strlen(p)+1] = '\0'; // null-terminated block of null-terminated strings
  p = strrchr (_pariExePath, DIR_SEPARATOR);  lstrcpy (p+1, PARI_EXE);

  BOOL r = CreateProcess (
              NULL,          // LPCTSTR lpApplicationName,
              _pariExePath,  // LPTSTR lpCommandLine
              NULL,          // process attributes

              NULL,          // thread attributes
#ifdef DAS_FUNKTIONIERT_NUR_UNTER_WINDOWS_98 //only Win98
              FALSE,         // inherit handles
              // CREATE_NEW_CONSOLE |
              HIGH_PRIORITY_CLASS |
              CREATE_NEW_PROCESS_GROUP, // creation flags
#else
              TRUE,          // inherit handles
              HIGH_PRIORITY_CLASS, // creation flags
#endif
              pariEnv,
              pariCurDir,    // current directory
              &si,
              &_processInfo
           );

  p[1] = '\0'; // _pariExePath inclusive dir-separator

  // we don't need the Handles on the PARI-side anymore
  CloseHandle (hPipeStdInRead);
  CloseHandle (hPipeStdOutWrite);
  CloseHandle (hPipeStdErrWrite);
  return r != 0;
}

void GpController::sendCtrlC() {
  // that doesn't work !!!
  GenerateConsoleCtrlEvent (CTRL_C_EVENT, _processInfo.dwProcessId);
  writeStdIn ("\x03"); // ^C
  HWND hPariWnd = FindWindow (NULL, "pari217");
  if (hPariWnd) {
    SendMessage (hPariWnd, WM_KEYDOWN, VK_CONTROL, 0);
    SendMessage (hPariWnd, WM_KEYDOWN, 'C',        0);
    //Sleep (100);
    SendMessage (hPariWnd, WM_KEYUP,   'C',        0);
    SendMessage (hPariWnd, WM_KEYUP,   VK_CONTROL, 0);
  }
}
#endif

#ifdef _LINUX
bool GpController::CreatePariChildProcess() {
  pipe(_pipeIn); pipe(_pipeOut); pipe(_pipeErr);
  _hPipeStdInWrite = _pipeIn[1]; hPipeStdInWrite = _pipeIn[0];
  _hPipeStdOutRead = _pipeOut[0]; hPipeStdOutRead = _pipeOut[1];
  _hPipeStdErrRead = _pipeErr[0]; hPipeStdErrRead = _pipeErr[1];

  signal(SIGCHLD, SIG_IGN); //no zombies, please
  _paripid = fork();
  if (_paripid < 0) return(0);

  else if (_paripid == 0) { //child
    _paripid = fork();
    if (_paripid <0) return(0);

    else if (_paripid > 0) {//child2, grandpa
      sleep(1000);
      exit(0);
    }

    else { //grandchild
      close (_hPipeStdInWrite); close(_hPipeStdOutRead); close(_hPipeStdErrRead);

      if (hPipeStdInWrite != fileno(stdin)) {  //connect
        if (dup2(hPipeStdInWrite, fileno(stdin)) != fileno(stdin)) return(0);
          close (hPipeStdInWrite);
      }

      if (hPipeStdOutRead != fileno(stdout)) {
        if (dup2(hPipeStdOutRead, fileno(stdout)) != fileno(stdout)) return(0);
          close (hPipeStdOutRead);
      }

      if (hPipeStdErrRead != fileno(stderr)) {
        if (dup2(hPipeStdErrRead, fileno(stderr)) != fileno(stderr)) return(0);
          close (hPipeStdErrRead);
      }

      if ((execlp("gp","gp","-q", (char *)0)) < 0)//start PARI in quit mode
        return(0);
      else return (1);
    }
   //parent
  close (hPipeStdInWrite); close(hPipeStdOutRead); close(hPipeStdErrRead);
  waitpid(_paripid, NULL, 0);
  return(1);
  }
}

void GpController::sendCtrlC() {
  sleep(100);
  writeStdIn ("\x03"); // ^C
}
#endif

#ifdef _WIN32
bool GpController::outputAvailable (HANDLE h) const {
  DWORD bytesAvail = 0;
  PeekNamedPipe (h,  NULL, 0, NULL, &bytesAvail, NULL);
  return bytesAvail > 0;
}

bool GpController::outputAvailable (OUTPUT o) const {
  DWORD bytesAvail = 0;
  HANDLE h = (o==STDOUT) ? _hPipeStdOutRead : _hPipeStdErrRead;
  PeekNamedPipe (h,  NULL, 0, NULL, &bytesAvail, NULL);
  return bytesAvail > 0;
}

const char* GpController::readOutput (HANDLE h) {
  // ATTENTION: waits until output from PARI appears!
  DWORD dwRead;
  ReadFile (h, _buffer, BUFSIZE, &dwRead, NULL);
  _buffer[dwRead] = '\0';
  return _buffer;
}

const char* GpController::readOutput (OUTPUT o) {
  // ATTENTION: waits until output from PARI appears!
  DWORD dwRead;
  HANDLE h = (o==STDOUT) ? _hPipeStdOutRead : _hPipeStdErrRead;
  ReadFile (h, _buffer, BUFSIZE, &dwRead, NULL);
  _buffer[dwRead] = '\0';
  return _buffer;
}

void GpController::writeStdIn (const char* text) {
  DWORD dwWritten;
  WriteFile (_hPipeStdInWrite, text, strlen(text),  &dwWritten, NULL);
}
#endif

bool GpController::stdOutAvailable() const {return outputAvailable(_hPipeStdOutRead);}
bool GpController::stdErrAvailable() const {return outputAvailable(_hPipeStdErrRead);}

const char* GpController::readStdOut() {return readOutput (_hPipeStdOutRead);}
const char* GpController::readStdErr() {return readOutput (_hPipeStdErrRead);}

#ifdef _LINUX
bool GpController::outputAvailable (int h) const {
  fd_set fd_read;
  struct timeval timevalue;
  timevalue.tv_sec = 0; timevalue.tv_usec = 100000;

  FD_ZERO(&fd_read);
  FD_SET(_hPipeStdOutRead,&fd_read);
  FD_SET(_hPipeStdErrRead,&fd_read);

  if ((select(16, &fd_read, NULL, NULL, &timevalue) > 0) && (FD_ISSET(h, &fd_read)))
    return(1);
  else return(0);
}

bool GpController::outputAvailable (OUTPUT o) const {
  fd_set fd_read;
  int h;
  struct timeval timevalue;
  timevalue.tv_sec = 0; timevalue.tv_usec = 100000;
  FD_ZERO(&fd_read);

  if(o == STDOUT)
    h = _hPipeStdOutRead;
  if(o == STDERR)
    h = _hPipeStdErrRead;

  FD_SET(_hPipeStdOutRead,&fd_read);
  FD_SET(_hPipeStdErrRead,&fd_read);

  if ((select(16, &fd_read, NULL, NULL, &timevalue) > 0) && (FD_ISSET(_handle, &fd_read)))
    return(1);
  else return(0);
}

const char* GpController::readOutput (int h) {
  // ATTENTION: waits until output from PARI appears!
  _number = read(h, _buffer, MAX_BUFF);
  _buffer[_number] = '\0';
  return _buffer;
}

const char* GpController::readOutput (OUTPUT o) {
  // ATTENTION: waits until output from PARI appears!
  if(o == STDOUT)
    _handle = hPipeStdOutRead;
  if(o == STDERR)
    _handle = hPipeStdErrRead;
  _number = read(_handle, _buffer, MAX_BUFF);
  _buffer[_number] = '\0';
  return _buffer;
}

void GpController::writeStdIn (const char* text) {
  _number = strlen(text);
  write(_hPipeStdInWrite, text, _number);
}
#endif

// operating-system dependent static methods
#ifdef _WIN32
bool GpController::isBreakHotkeyPressed() {
  return GetAsyncKeyState(VK_MENU) < 0 
      && GetAsyncKeyState(VK_CONTROL) < 0;  // Alt and Ctrl pressed
}

bool GpController::copyFile (const char* source, const char* target) {
  // if the target exist, it will deleted without further queries!
  return CopyFile (source, target, FALSE) != 0;
}

bool GpController::deleteFile (const char* file) {
  return DeleteFile (file) != 0;
}

void GpController::sleep (int milliseconds) {
  ::Sleep (milliseconds);
}
#endif

#ifdef _LINUX
bool GpController::isBreakHotkeyPressed() { //TODO
  bool _breakHotkeyPressed;
  return(_breakHotkeyPressed);
}

void GpController::setBreakHotkeyPressed(bool state){
  bool _breakHotkeyPressed;
 _breakHotkeyPressed = state;
}

bool GpController::copyFile (const char* source, const char* target) {
// if the target exist, it will deleted without further queries!
  FILE *sourcefp, *targetfp;
  int h = 0;
  if ((sourcefp = fopen(source, "r")) == NULL)
    return(0);
  if ((targetfp = fopen(target, "w")) == NULL)
    return(0);
  while ((h = fgetc(sourcefp)) != EOF)
    putc(h, targetfp);
  fflush(targetfp);
  fclose(sourcefp);
  fclose(targetfp);
  return(1);
}

bool GpController::deleteFile (const char* file) {
  if ((unlink(file)) != 0)
    return(0);
  return(1);
}

void GpController::sleep (int milliseconds) {
  usleep (milliseconds);
}
#endif
