/**************************************************************************
 *  Copyright (c) 2002  Hartmut Ring, Universitaet Siegen                 *
 *  http://www.math.uni-siegen.de/ring                                    *
 *                                                                        *
 *  This file is part of the pariGUIde package.                           *
 *                                                                        *
 *  pariGUIde is free software; you can redistribute it and/or modify it  *
 *  under the terms of the GNU General Public License (GPL)               *
 *  as published by the Free Software Foundation:                         *
 *  http://www.opensource.org/licenses/gpl-license.html                   *
 *************************************************************************/
// gpcontroller.h
//
// communication with PARI-GP
// implemetation and privat part of the class depends on the operating-system!

#ifndef __GPCONTROLLER_H__
#define __GPCONTROLLER_H__

#ifdef _WIN32
#define DIR_SEPARATOR        '\\'
#define DIR_SEPARATOR_STRING "\\"
#define PARI_EXE             "Pari217.exe"
#include <windows.h>
#endif

#ifdef _LINUX
#define DIR_SEPARATOR '/'
#define DIR_SEPARATOR_STRING  "/"
#define PARI_EXE  "gp"
#define MAX_PATH  4000  //4096
#define MAX_BUFF  4000 //4096

#include <stdio.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/wait.h>
#include <fcntl.h>
#include <signal.h>
#include <string.h>
#include <stdlib.h>
#endif

class GpController {
public:
  enum OUTPUT {STDOUT, STDERR};
  GpController();
  virtual ~GpController();

  // send input to the stdin of PARI
  void writeStdIn (const char* text); 

  bool outputAvailable (OUTPUT o) const;
  const char* readOutput (OUTPUT o);

  // fetch the ouput from stdout of PARI
  bool stdOutAvailable() const;
  const char* readStdOut(); // expires after the next readStdOut() or readStdErr()

  // fetch the output from stderr of PARI
  bool stdErrAvailable() const;
  const char* readStdErr(); // expires after the next readStdOut() or readStdErr()

  void sendCtrlC(); // interrupts the calculation in PARI

  // program-path inclusive directory-decollator
  const char* programPath() const {return _pariExePath;}

  static bool isBreakHotkeyPressed();
  static void setBreakHotkeyPressed(bool state);

  static bool copyFile (const char* source, const char* target);
  static bool deleteFile (const char* file);
  static void sleep (int milliseconds);

private:
  bool CreatePariChildProcess();
  char _pariExePath[MAX_PATH];
  bool  _breakHotkeyPressed;

#ifdef _WIN32
  bool outputAvailable (HANDLE h) const;
  const char* readOutput (HANDLE h);

  HANDLE _hPipeStdOutRead; // stdout from Pari
  HANDLE _hPipeStdErrRead; // stderr from Pari
  HANDLE _hPipeStdInWrite; // stdin  from Pari
  PROCESS_INFORMATION _processInfo;
#endif

#ifdef _LINUX
  bool outputAvailable (int h) const;
  const char* readOutput (int h);

  int _number, _handle;
  pid_t _paripid;

//  char line[MAX_BUFF];

  int _hPipeStdOutRead, _hPipeStdErrRead, _hPipeStdInWrite;
  int hPipeStdOutRead, hPipeStdErrRead, hPipeStdInWrite;
  int _pipeIn[2]; // stdout from Pari
  int _pipeErr[2]; // stderr from Pari
  int _pipeOut[2]; // stdin  from Pari

  char _processInfo[MAX_BUFF];
#endif

  enum {BUFSIZE = MAX_BUFF};
  char* _buffer;
};

#endif //__GPCONTROLLER_H__
