/**************************************************************************
 *  Copyright (c) 2002  Hartmut Ring, Universitaet Siegen                 *
 *  http://www.math.uni-siegen.de/ring                                    *
 *                                                                        *
 *  This file is part of the pariGUIde package.                           *
 *                                                                        *
 *  pariGUIde is free software; you can redistribute it and/or modify it  *
 *  under the terms of the GNU General Public License (GPL)               *
 *  as published by the Free Software Foundation:                         *
 *  http://www.opensource.org/licenses/gpl-license.html                   *
 *************************************************************************/
// pariGUIwindow.cpp

#include "pariGUIwindow.h"
#include "editwindow.h"
#include "gpcontroller.h"

#include <qpixmap.h>
#include <qtoolbar.h>
#include <qtoolbutton.h>
#include <qpopupmenu.h>
#include <qmenubar.h>
#include <qstatusbar.h>
#include <qapplication.h>
#include <qtimer.h>
#include <qmessagebox.h>

#include "xpm/fileopen.xpm"
#include "xpm/filesave.xpm"
#include "xpm/calc.xpm"
//#include "xpm/stop.xpm"
#include "xpm/step.xpm"
#include "xpm/up.xpm"
#include "xpm/down.xpm"
#include "xpm/sumbtn.xpm"
#include "xpm/prodbtn.xpm"
#include "xpm/matrix.xpm"
#include "xpm/table.xpm"
#include "xpm/plot.xpm"
#include "xpm/intnum.xpm"

PariGuiWindow::PariGuiWindow()
  : QMainWindow (0, "main window", WDestructiveClose)
{
  _pariGP = new GpController();
  setCaption("pariGUIde");

  // work window
  _editWnd = new EditWindow (this, _pariGP);
  setCentralWidget (_editWnd);

  // tool bar
  _toolbar = new QToolBar (this, "toolbar");
  _toolbar->setLabel (tr("Toolbar"));

  QPixmap iconOpen   = QPixmap (fileopen);
  QPixmap iconSave   = QPixmap (filesave);
  QPixmap iconStep   = QPixmap (imgStep);

  QPixmap iconCalc   = QPixmap (imgCalc);
  QPixmap iconUp     = QPixmap (imgUp);
  QPixmap iconDown   = QPixmap (imgDown);
//  QPixmap iconStop   = QPixmap (imgStop);

  QPixmap iconSumBtn = QPixmap (sumbtn);
  QPixmap iconProdBtn= QPixmap (prodbtn);
  QPixmap iconMatrix = QPixmap (xpmMatrix);
  QPixmap iconTable  = QPixmap (xpmTable);
  QPixmap iconPlot   = QPixmap (imgPlot);
  QPixmap iconIntnum  = QPixmap (intnum);

/////////
  _stepAction = new QAction (tr("Next step of the presentation"), QIconSet(iconStep),
                             tr("&Next step"), CTRL+Key_N, this);
  _stepAction->setEnabled (false);
  connect (_stepAction, SIGNAL(activated()), this, SLOT(demoStep()));

  QAction* copyAction = new QAction (tr("Copy previous input from the protocol"), QIconSet(iconUp),
                                   tr("Previous input"), ALT+Key_Up, this);
  connect (copyAction, SIGNAL(activated()), _editWnd, SLOT(up()));

  QAction* downAction = new QAction (tr("Copy next input from the protocol"), QIconSet(iconDown),
                                   tr("Next input"), ALT+Key_Down, this);
  connect (downAction, SIGNAL(activated()), _editWnd, SLOT(down()));
/*
  QAction* stopAction = new QAction (tr("Stop the calculation (not implemented yet)"), QIconSet(iconStop),
                                   tr("&Cancel"), ALT+CTRL+Key_C, this);
  connect (stopAction, SIGNAL(activated()), _editWnd, SLOT(stop()));
*/
  QAction* sumAction = new QAction (tr("Define sum"), QIconSet(iconSumBtn),
                                   tr("&Sum..."), SHIFT+CTRL+Key_S, this);
  connect (sumAction, SIGNAL(activated()), _editWnd, SLOT(insertSum()));

  QAction* prodAction = new QAction (tr("Define product"), QIconSet(iconProdBtn),
                                   tr("&Product..."), SHIFT+CTRL+Key_P, this);
  connect (prodAction, SIGNAL(activated()), _editWnd, SLOT(insertProd()));

  QAction* matrixAction = new QAction (tr("Define matrix"), QIconSet(iconMatrix),
                                   tr("&Matrix..."), SHIFT+CTRL+Key_M, this);
  connect (matrixAction, SIGNAL(activated()), _editWnd, SLOT(insertMatrix()));

  QAction* tableAction = new QAction (tr("Define value-table"), QIconSet(iconTable),
                                   tr("Value &table..."), SHIFT+CTRL+Key_T, this);
  connect (tableAction, SIGNAL(activated()), _editWnd, SLOT(insertTable()));

  QAction* plotAction = new QAction (tr("Define graph"), QIconSet(iconPlot),
                                   tr("&Graph..."), SHIFT+CTRL+Key_G, this);
  connect (plotAction, SIGNAL(activated()), _editWnd, SLOT(insertPlot()));

  QAction* intAction = new QAction (tr("Numeric integrate"), QIconSet(iconIntnum),
                                  tr("&Integrate..."), SHIFT+CTRL+Key_I, this);
  connect (intAction, SIGNAL(activated()), _editWnd, SLOT(insertInt()));
///////

  QToolButton* btnFileOpen = new QToolButton (iconOpen, tr("Open file (Ctrl+O)"),
                                              tr("Open Pari-GP file"),
                                              _editWnd, SLOT(open()), _toolbar);

  QToolButton* btnFileSave = new QToolButton (iconSave, tr("Save as HTML (Ctrl+S)"),
                                              QString::null,
                                              _editWnd, SLOT(save()), _toolbar);

  _stepAction->addTo (_toolbar);
  _toolbar->addSeparator();

  QToolButton* btnFileCalc = new QToolButton (iconCalc, tr("Compute (Ctrl+Return)"),
                                              QString::null,
                                              _editWnd, SLOT(calc()), _toolbar);

  copyAction->addTo (_toolbar);
  downAction->addTo (_toolbar);
//  stopAction->addTo (_toolbar);
  _toolbar->addSeparator();

  sumAction->addTo (_toolbar);
  prodAction->addTo (_toolbar);
  matrixAction->addTo (_toolbar);
  tableAction->addTo (_toolbar);
  plotAction->addTo (_toolbar);
  intAction-> addTo (_toolbar);

  // Menu
  QPopupMenu* menuPresentation = new QPopupMenu (this);
  menuPresentation->insertItem (tr("Open (HTML-protocol)..."), this, SLOT(openDemo()));
  _stepAction->addTo (menuPresentation);

  QPopupMenu* menuFile = new QPopupMenu (this);

  menuBar()->insertItem (tr("&File"), menuFile);
  menuFile->insertItem (iconOpen, tr("&Open..."), _editWnd, SLOT(open()), CTRL+Key_O);
  menuFile->insertItem (iconSave, tr("&Save HTML..."), _editWnd, SLOT(save()), CTRL+Key_S);
  menuFile->insertItem (tr("&Presentation"), menuPresentation);
  menuFile->insertSeparator();
  menuFile->insertItem (tr("&Quit"), this, SLOT(quit()), CTRL+Key_Q);


  QPopupMenu* menuEdit = new QPopupMenu (this);
  menuBar()->insertItem (tr("&Edit"), menuEdit);
  menuEdit->insertItem (iconCalc, tr("Compute"), _editWnd, SLOT(calc()), CTRL+Key_Return);
  copyAction->addTo (menuEdit);
  downAction->addTo (menuEdit);
//  stopAction->addTo (menuEdit);

  QPopupMenu* menuInsert = new QPopupMenu (this);
  menuBar()->insertItem (tr("&Paste"), menuInsert);
  //menuInsert->insertItem ("&Summe...", _editWnd, SLOT(insertSum()), SHIFT+CTRL+Key_S);
  sumAction->addTo (menuInsert);
  //menuInsert->insertItem ("&Produkt...", _editWnd, SLOT(insertProd()), SHIFT+CTRL+Key_P);

  prodAction->addTo (menuInsert);
  //menuInsert->insertItem ("&Matrix...", _editWnd, SLOT(insertMatrix()), SHIFT+CTRL+Key_M);
  matrixAction->addTo (menuInsert);
  //menuInsert->insertItem ("&Wertetafel...", _editWnd, SLOT(insertTable()), SHIFT+CTRL+Key_W);
  tableAction->addTo (menuInsert);
  plotAction->addTo (menuInsert);
  intAction-> addTo (menuInsert);

  menuInsert->insertSeparator();
  menuInsert->insertItem (tr("&Alternative (If)"), _editWnd, SLOT(insertIf()), SHIFT+CTRL+Key_A);

  QPopupMenu* menuOptions = new QPopupMenu (this);
  menuBar()->insertItem (tr("&Options"), menuOptions);
  menuOptions->insertItem (tr("&Computational accuracy.."), _editWnd, SLOT(setPrecision()), CTRL+Key_P);
  menuOptions->insertItem (tr("&Font..."), _editWnd, SLOT(selectFont()));

  QPopupMenu* menuHelp = new QPopupMenu (this);
  menuBar()->insertItem (tr("&Help"), menuHelp);
  menuHelp->insertItem (tr("About pariGUIde"), this, SLOT(about()));
//  menuHelp->insertItem(tr("User Manual"), this, SLOT(manual()));
  statusBar()->message (tr("ready"), 2000);
}

PariGuiWindow::~PariGuiWindow() {
  if (_pariGP) {
    delete _pariGP;
    _pariGP = 0;
  }
}

void PariGuiWindow::about() {
  QMessageBox::about (this, tr("About pariGUIde"),
    "pariGUIde version " PARI_GUI_VERSION "\n"
    "Copyright (c) 2002  Hartmut Ring (Universitt Siegen):\n"
    "ring@math.uni-siegen.de\n"
    "Linux port and some contributions from Henning Westerholt:\n"
    "henning@skalatan.de\n\n"
    "This program use GP/PARI CALCULATOR as server\n"
    "(Copyright (c) 1989-1999 by C. Batut, K. Belabas, D. Bernardi, H. Cohen and M. Olivier)\n\n"
    "This is free software, i.e. it is distributed free of charge, including the complete source code.\n"
    "Users have the right to copy and modify this software under the same terms."
  );
}
                                  
void PariGuiWindow::quit() {
  if (_pariGP) {
    delete _pariGP;
    _pariGP = 0;
  }
  qApp->quit();
}

void PariGuiWindow::closeEvent (QCloseEvent* ce) {
  if (_pariGP) {
    delete _pariGP;
    _pariGP = 0;
  }
  ce->accept(); // TODO provide save option
//  ce->ignore();
}

void PariGuiWindow::openDemo() {
  _stepAction->setEnabled (_editWnd->openDemo());
}

void PariGuiWindow::demoStep() {
  _stepAction->setEnabled (_editWnd->demoStep());
}
